/*
 * acast.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <string>
#include <sax/FromXMLParserHelper.h>
#include <exception/CommonException.h>
#include <factory/XmlDataFactory.hpp>
#include <object/Object.h>
#include <cast/CastApi.hpp>

// -----------------------------------------------------------------------------------------

int main(int argc, char** argv) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("cast binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to cast",	false,	"-",		"file");
		cmd.add( input );

		TCLAP::ValueArg<std::string> type(	"t",	"type",		"Output type",		true,	"",		"string");
		cmd.add( type );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		alib::Object in = alib::XmlDataFactory::fromTokens<alib::Object>(sax::FromXMLParserHelper::parseInput(input));

		measurements::end();
		measurements::start("Cast", measurements::Type::MAIN);

		alib::Object res = alib::castApi::getCastPool(type.getValue()).cast(in.getData());

		measurements::end();
		measurements::start("Output write", measurements::Type::AUXILIARY);

		alib::XmlDataFactory::toStdout(res);

		measurements::end();
		measurements::end();

		if(measure.getValue()) std::cmeasure << measurements::results() << std::endl;

		return 0;
	} catch(const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
