/*
 * ExactSubtreeRepeatsNaive.cpp
 *
 *  Created on: 1. 4. 2016
 *      Author: Jan Travnicek
 */

#include "ExactSubtreeRepeatsNaive.h"
#include "SubtreeJumpTable.h"

#include <tree/ranked/RankedTree.h>
#include <tree/ranked/PrefixRankedTree.h>
#include <tree/ranked/PrefixRankedBarTree.h>
#include <tree/Tree.h>
#include <global/GlobalData.h>

namespace arbology {

namespace exact {

tree::Tree ExactSubtreeRepeatsNaive::repeats ( const tree::Tree & tree ) {
	return dispatch ( tree.getData ( ) );
}

std::tree < alphabet::RankedSymbol > ExactSubtreeRepeatsNaive::repeats ( const std::tree < alphabet::RankedSymbol > & node, std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > & data, int & minId ) {
	std::vector < std::tree < alphabet::RankedSymbol > > children;
	std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > > childRepeatsKey ( node.getData ( ), std::vector < alphabet::RankedSymbol > ( ) );

	for ( const std::tree < alphabet::RankedSymbol > & child : node.getChildren() ) {
		children.push_back ( repeats ( child, data, minId ) );
		childRepeatsKey.second.push_back ( children.back ( ).getData ( ) );
	}

	int & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	return std::tree < alphabet::RankedSymbol > ( alphabet::RankedSymbol ( alphabet::symbolFrom ( uniqueRepeatId ), node.getData ( ).getRank ( ) ), std::move ( children ) );
}

tree::RankedTree ExactSubtreeRepeatsNaive::repeats ( const tree::RankedTree & tree ) {
	int minId = 1;
	std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > data;

	return tree::RankedTree ( repeats ( tree.getContent ( ), data, minId ) );
}

auto ExactRepeatsNaiveRankedTree = ExactSubtreeRepeatsNaive::RegistratorWrapper < tree::RankedTree, tree::RankedTree > ( ExactSubtreeRepeatsNaive::repeats );

alphabet::RankedSymbol ExactSubtreeRepeatsNaive::repeatsPrefixRanked ( const std::vector < alphabet::RankedSymbol > & symbols, std::vector < alphabet::RankedSymbol > & res, std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > & data, int & minId, int & index ) {
	int begin = index;
	std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > > childRepeatsKey ( symbols[begin], std::vector < alphabet::RankedSymbol > ( ) );

	res.push_back ( alphabet::RankedSymbol ( alphabet::symbolFrom ( 0 ), symbols[begin].getRank ( ) ) );

	index++;

	for ( unsigned i = 0; i < symbols[begin].getRank ( ).getData ( ); ++i )
		childRepeatsKey.second.push_back ( repeatsPrefixRanked ( symbols, res, data, minId, index ) );

	int & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	res[begin] = alphabet::RankedSymbol ( alphabet::symbolFrom ( uniqueRepeatId ), symbols[begin].getRank ( ) );
	return res[begin];
}

tree::PrefixRankedTree ExactSubtreeRepeatsNaive::repeats ( const tree::PrefixRankedTree & tree ) {
	int minId = 1;
	int index = 0;
	std::vector < alphabet::RankedSymbol > res;
	std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > data;

	repeatsPrefixRanked ( tree.getContent ( ), res, data, minId, index );
	return tree::PrefixRankedTree ( res );
}

auto ExactRepeatsNaivePrefixRankedTree = ExactSubtreeRepeatsNaive::RegistratorWrapper < tree::PrefixRankedTree, tree::PrefixRankedTree > ( ExactSubtreeRepeatsNaive::repeats );

alphabet::RankedSymbol ExactSubtreeRepeatsNaive::repeatsPrefixRankedBar ( const std::vector < alphabet::RankedSymbol > & symbols, std::vector < alphabet::RankedSymbol > & res, std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > & data, int & minId, int & index ) {
	int begin = index;
	std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > > childRepeatsKey ( symbols[begin], std::vector < alphabet::RankedSymbol > ( ) );

	res.push_back ( alphabet::RankedSymbol ( alphabet::symbolFrom ( 0 ), symbols[begin].getRank ( ) ) );

	index++;

	for ( unsigned i = 0; i < symbols[begin].getRank ( ).getData ( ); ++i )
		childRepeatsKey.second.push_back ( repeatsPrefixRankedBar ( symbols, res, data, minId, index ) );

	int & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	res[begin] = alphabet::RankedSymbol ( alphabet::symbolFrom ( uniqueRepeatId ), symbols[begin].getRank ( ) );
	res.push_back ( symbols[index] );
	index++;

	return res[begin];
}

tree::PrefixRankedBarTree ExactSubtreeRepeatsNaive::repeats ( const tree::PrefixRankedBarTree & tree ) {
	int minId = 1;
	int index = 0;
	std::vector < alphabet::RankedSymbol > res;
	std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > data;

	repeatsPrefixRankedBar ( tree.getContent ( ), res, data, minId, index );
	return tree::PrefixRankedBarTree ( tree.getBars ( ), res );
}

auto ExactRepeatsNaivePrefixRankedBarTree = ExactSubtreeRepeatsNaive::RegistratorWrapper < tree::PrefixRankedBarTree, tree::PrefixRankedBarTree > ( ExactSubtreeRepeatsNaive::repeats );

} /* namespace exact */

} /* namespace arbology */
