/*
 * Occurrences.cpp
 *
 *  Created on: 9. 2. 2014
 *	  Author: Jan Travnicek
 */

#include "Occurrences.h"
#include "Run.h"
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/DPDA.h>

#include <deque>

namespace automaton {

namespace run {

std::set < unsigned > Occurrences::occurrences ( const automaton::Automaton & automaton, const alib::Object & object ) {
	return dispatch ( automaton.getData ( ), object.getData ( ) );
}

std::set < unsigned > Occurrences::occurrences ( const automaton::Automaton & automaton, const string::LinearString < > & string ) {
	return dispatch ( automaton.getData ( ), string );
}

std::set < unsigned > Occurrences::occurrences ( const automaton::Automaton & automaton, const tree::RankedTree & tree ) {
	return dispatch ( automaton.getData ( ), tree );
}

std::set < unsigned > Occurrences::occurrences ( const automaton::DFA < > & automaton, const string::LinearString < > & string ) {
	std::tuple < bool, label::Label, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesDFALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::DFA<>, string::LinearString < > > ( Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::DFTA & automaton, const tree::RankedTree & tree ) {
	std::tuple < bool, label::Label, std::set < unsigned > > res = Run::calculateState ( automaton, tree );

	return std::get < 2 > ( res );
}

auto OccurrencesDFTARankedTree = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::DFTA, tree::RankedTree > ( Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::InputDrivenDPDA < > & automaton, const string::LinearString < > & string ) {
	std::tuple < bool, label::Label, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesInputDrivenDPDALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::InputDrivenDPDA < >, string::LinearString < > > ( Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::VisiblyPushdownDPDA < > & automaton, const string::LinearString < > & string ) {
	std::tuple < bool, label::Label, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesVisiblyPushdownDPDALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::VisiblyPushdownDPDA < >, string::LinearString < > > ( Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::RealTimeHeightDeterministicDPDA < > & automaton, const string::LinearString < > & string ) {
	std::tuple < bool, label::Label, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesRealTimeHeightDeterministicDPDALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::RealTimeHeightDeterministicDPDA < >, string::LinearString < > > ( Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::DPDA < > & automaton, const string::LinearString < > & string ) {
	std::tuple < bool, label::Label, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesDPDALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::DPDA < >, string::LinearString < > > ( Occurrences::occurrences );

} /* namespace run */

} /* namespace automaton */
