/*
 * RankedSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef RANKED_SYMBOL_H_
#define RANKED_SYMBOL_H_

#include "Symbol.h"
#include <primitive/Unsigned.h>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class RankedSymbol : public SymbolBase {
protected:
	alphabet::Symbol symbol;
	primitive::Unsigned rank;

public:
	explicit RankedSymbol ( int number, int rank );
	explicit RankedSymbol ( char character, int rank );
	explicit RankedSymbol ( std::string label, int rank );

	/**
	 * Creates new symbol with given name and rank.
	 * @param symbol name of the symbol
	 * @param rank of the symbol
	 */
	explicit RankedSymbol ( alphabet::Symbol symbol, primitive::Unsigned rank );
	explicit RankedSymbol ( alphabet::Symbol symbol, int rank );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const alphabet::Symbol & getSymbol ( ) const;

	/**
	 * @return rank of the symbol
	 */
	const primitive::Unsigned & getRank ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const RankedSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "RankedSymbol";

		return xmlTagName;
	}

	static RankedSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* RANKED_SYMBOL_H_ */
