/*
 * AutomatonFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_FEATURES_H_
#define AUTOMATON_FEATURES_H_

#include <alphabet/Symbol.h>
#include <label/Label.h>
#include <string/Epsilon.h>

namespace automaton {

enum class FEATURES {
	EPSILON_NFA,
	MULTI_INITIAL_STATE_NFA,
	NFA,
	DFA,
	COMPACT_NFA,
	EXTENDED_NFA,
	DPDA,
	SINGLE_POP_DPDA,
	INPUT_DRIVEN_DPDA,
	INPUT_DRIVEN_NPDA,
	VISIBLY_PUSHDOWN_DPDA,
	VISIBLY_PUSHDOWN_NPDA,
	REAL_TIME_HEIGHT_DETERMINISTIC_DPDA,
	REAL_TIME_HEIGHT_DETERMINISTIC_NPDA,
	NPDA,
	SINGLE_POP_NPDA,
	ONE_TAPE_DTM,
	DFTA,
	NFTA
};

class Automaton;
class AutomatonBase;
enum class Shift;

template<class SymbolType = typename alphabet::Symbol, class EpsilonType = typename string::Epsilon < SymbolType >, class StateType = typename label::Label >
class EpsilonNFA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class NFA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class MultiInitialStateNFA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class DFA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class CompactNFA;
class ExtendedNFA;
template<class SymbolType = typename alphabet::Symbol, class EpsilonType = typename string::Epsilon < SymbolType >, class StateType = typename label::Label >
class DPDA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class InputDrivenDPDA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class InputDrivenNPDA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class VisiblyPushdownDPDA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class VisiblyPushdownNPDA;
template<class SymbolType = typename alphabet::Symbol, class EpsilonType = typename string::Epsilon < SymbolType >, class StateType = typename label::Label >
class RealTimeHeightDeterministicDPDA;
template<class SymbolType = typename alphabet::Symbol, class EpsilonType = typename string::Epsilon < SymbolType >, class StateType = typename label::Label >
class RealTimeHeightDeterministicNPDA;
template<class SymbolType = typename alphabet::Symbol, class EpsilonType = typename string::Epsilon < SymbolType >, class StateType = typename label::Label >
class NPDA;
template<class SymbolType = typename alphabet::Symbol, class EpsilonType = typename string::Epsilon < SymbolType >, class StateType = typename label::Label >
class NPDTA;
template<class SymbolType = typename alphabet::Symbol, class EpsilonType = typename string::Epsilon < SymbolType >, class StateType = typename label::Label >
class SinglePopDPDA;
template<class SymbolType = typename alphabet::Symbol, class EpsilonType = typename string::Epsilon < SymbolType >, class StateType = typename label::Label >
class SinglePopNPDA;
template<class SymbolType = typename alphabet::Symbol, class StateType = typename label::Label >
class OneTapeDTM;
class DFTA;
class NFTA;

} /* namespace automaton */

#endif /* AUTOMATON_FEATURES_H_ */
