/*
 * Tree.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "ObjectsTree.h"

#include "Container.h"
#include <object/Object.h>
#include <sstream>

namespace container {

ObjectsTree::ObjectsTree ( std::tree < alib::Object > tree ) : std::tree < alib::Object > ( std::move ( tree ) ) {
}

ContainerBase * ObjectsTree::clone ( ) const {
	return new ObjectsTree ( * this );
}

ContainerBase * ObjectsTree::plunder ( ) && {
	return new ObjectsTree ( std::move ( * this ) );
}

int ObjectsTree::compare ( const ObjectsTree & other ) const {
	std::compare < std::tree < alib::Object > > comp;

	return comp ( static_cast < const std::tree < alib::Object > > ( * this ), static_cast < const std::tree < alib::Object > > ( other ) );
}

void ObjectsTree::operator >>( std::ostream & os ) const {
	os << "(ObjectsTree " << static_cast < const std::tree < alib::Object > > ( * this ) << ")";
}

ObjectsTree::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

ObjectsTree ObjectsTree::parse ( std::deque < sax::Token >::iterator & input ) {
	return ObjectsTree ( parseRaw < alib::Object > ( input ) );
}

void ObjectsTree::compose ( std::deque < sax::Token > & out ) const {
	compose ( out, * this );
}

} /* namespace container */

namespace alib {

auto objectsTreeParserRegister = xmlApi < container::Container >::ParserRegister < container::ObjectsTree > ( );
auto objectsTreeParserRegister2 = xmlApi < alib::Object >::ParserRegister < container::ObjectsTree > ( );

} /* namespace alib */
