/*
 * IndexFromXMLParser.cpp
 *
 *  Created on: Nov 16, 2014
 *      Author: Jan Travnicek
 */

#include <sax/FromXMLParserHelper.h>
#include "IndexFromXMLParser.h"
#include <sax/ParserException.h>
#include <XmlApi.hpp>
#include <primitive/Bool.h>

namespace indexes {

std::set < alphabet::Symbol > IndexFromXMLParser::parseAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > symbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		symbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );
	return symbols;
}

SuffixTrieNodeFinalMark * IndexFromXMLParser::parseSuffixTrieNodeFinalMark ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "node" );
	std::map < alphabet::Symbol, SuffixTrieNodeFinalMark * > children;
	bool finalMark = alib::xmlApi < bool >::parse ( input );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) ) {
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "child" );
		alphabet::Symbol symbol = alib::xmlApi < alphabet::Symbol >::parse ( input );
		children.insert ( std::make_pair ( std::move ( symbol ), parseSuffixTrieNodeFinalMark ( input ) ) );
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "child" );
	}

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "node" );

	return new SuffixTrieNodeFinalMark ( children, finalMark );
}

SuffixTrieNodeTerminatingSymbol * IndexFromXMLParser::parseSuffixTrieNodeTerminatingSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "node" );
	std::map < alphabet::Symbol, SuffixTrieNodeTerminatingSymbol * > children;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) ) {
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "child" );
		alphabet::Symbol symbol = alib::xmlApi < alphabet::Symbol >::parse ( input );
		children.insert ( std::make_pair ( std::move ( symbol ), parseSuffixTrieNodeTerminatingSymbol ( input ) ) );
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "child" );
	}

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "node" );

	return new SuffixTrieNodeTerminatingSymbol ( children );
}

} /* namespace indexes */
