/*
 * LR0ItemsLabel.cpp
 *
 *  Created on: 29. 4. 2016
 *      Author: Martin Kocicka
 */

#include "LR0ItemsLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include "UniqueLabel.h"
#include "../container/ObjectsMap.h"
#include "../container/ObjectsSet.h"
#include "../container/ObjectsVector.h"
#include <primitive/Unsigned.h>

namespace label {

LR0ItemsLabel::LR0ItemsLabel ( grammar::parsing::LR0Items items ) : items ( items ) {
}

LR0ItemsLabel::LR0ItemsLabel ( ) {
}

LabelBase* LR0ItemsLabel::clone ( ) const {
	return new LR0ItemsLabel ( *this );
}

LabelBase* LR0ItemsLabel::plunder ( ) && {
	return new LR0ItemsLabel ( std::move ( *this ) );
}

bool LR0ItemsLabel::operator < ( const LR0ItemsLabel & other ) const {
	return this->compare(other) < 0;
}

int LR0ItemsLabel::compare ( const LR0ItemsLabel & other ) const {
	if ( items < other.items ) {
		return -1;
	} else if ( items > other.items ) {
		return 1;
	} else {
		return 0;
	}
}

void LR0ItemsLabel::operator>>(std::ostream & out) const {
	out << "(LR0ItemsLabel " << items << ")";
}

LR0ItemsLabel::operator std::string ( ) const {
	std::stringstream ss;
	ss << *this;
	return ss.str ( );
}

LR0ItemsLabel LR0ItemsLabel::parse ( std::deque<sax::Token>::iterator& input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, LR0ItemsLabel::getXmlTagName() );
	LR0ItemsLabel data ( alib::xmlApi < grammar::parsing::LR0Items > ::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, LR0ItemsLabel::getXmlTagName() );
	return data;
}

void LR0ItemsLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back ( LR0ItemsLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT );
	alib::xmlApi < grammar::parsing::LR0Items > ::compose ( out, items );
	out.emplace_back ( LR0ItemsLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT );
}

LabelBase* LR0ItemsLabel::next ( ) const {
	return new UniqueLabel ( Label ( *this ), primitive::Integer ( 0 ) );
}

LabelBase* LR0ItemsLabel::inc ( ) && {
	return new UniqueLabel ( Label ( std::move ( *this ) ), primitive::Integer ( 0 ) );
}

} /* namespace label */

namespace alib {

auto lr0ItemsLabelParserRegister = xmlApi < label::Label > ::ParserRegister < label::LR0ItemsLabel > ( );
auto lr0ItemsLabelParserRegister2 = xmlApi < alib::Object > ::ParserRegister < label::LR0ItemsLabel > ( );

} /* namespace alib */
