/*
 * RTEFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RTEFromXMLParser.h"

#include "../RTE.h"
#include "../RTEClasses.h"

#include "../formal/FormalRTEElements.h"
#include <XmlApi.hpp>

namespace rte {

std::pair < std::set < alphabet::RankedSymbol >, std::set < alphabet::RankedSymbol > > RTEFromXMLParser::parseAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::RankedSymbol > alphabetF, alphabetK;

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabetF.insert ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbolAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabetK.insert ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substSymbolAlphabet" );

	return std::make_pair ( alphabetF, alphabetK );
}

std::rvalue_ref < FormalRTEElement > RTEFromXMLParser::parseFormalRTEElement ( std::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" ) )
		return parseFormalRTEEmpty ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" ) )
		return parseFormalRTEIteration ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" ) )
		return parseFormalRTEAlternation ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "substitution" ) )
		return parseFormalRTESubstitution ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" ) )
		return parseFormalRTESymbolAlphabet ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbol" ) )
		return parseFormalRTESymbolSubst ( input );
	else
		throw exception::CommonException ( "Invalid token" );
}

std::rvalue_ref < FormalRTEAlternation > RTEFromXMLParser::parseFormalRTEAlternation ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" );

	std::rvalue_ref < FormalRTEElement > element1 = parseFormalRTEElement ( input );
	std::rvalue_ref < FormalRTEElement > element2 = parseFormalRTEElement ( input );

	std::rvalue_ref < FormalRTEAlternation > alternation ( new FormalRTEAlternation ( std::move ( element1 ), std::move ( element2 ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alternation" );
	return alternation;
}

std::rvalue_ref < FormalRTESubstitution > RTEFromXMLParser::parseFormalRTESubstitution ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substitution" );

	std::rvalue_ref < FormalRTESymbolSubst > ssymb ( new FormalRTESymbolSubst ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) ) );
	std::rvalue_ref < FormalRTEElement > element1 = parseFormalRTEElement ( input );
	std::rvalue_ref < FormalRTEElement > element2 = parseFormalRTEElement ( input );

	std::rvalue_ref < FormalRTESubstitution > substitution ( new FormalRTESubstitution ( std::move ( element1 ), std::move ( element2 ), std::move ( ssymb ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substitution" );
	return substitution;
}

std::rvalue_ref < FormalRTEIteration > RTEFromXMLParser::parseFormalRTEIteration ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" );

	std::rvalue_ref < FormalRTESymbolSubst > ssymb ( new FormalRTESymbolSubst ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) ) );
	std::rvalue_ref < FormalRTEElement > element = parseFormalRTEElement ( input );
	std::rvalue_ref < FormalRTEIteration > iteration ( new FormalRTEIteration ( std::move ( element ), std::move ( ssymb ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "iteration" );
	return iteration;
}

std::rvalue_ref < FormalRTEEmpty > RTEFromXMLParser::parseFormalRTEEmpty ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" );

	std::rvalue_ref < FormalRTEEmpty > empty ( new FormalRTEEmpty ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "empty" );

	return empty;
}

std::rvalue_ref < FormalRTESymbol > RTEFromXMLParser::parseFormalRTESymbol ( std::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" ) )
		return parseFormalRTESymbolAlphabet ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbol" ) )
		return parseFormalRTESymbolSubst ( input );
	else
		throw exception::CommonException ( "Expected symbol or substSymbol" );
}

std::rvalue_ref < FormalRTESymbolAlphabet > RTEFromXMLParser::parseFormalRTESymbolAlphabet ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" );
	alphabet::RankedSymbol symbol = alib::xmlApi < alphabet::RankedSymbol >::parse ( input );
	std::vector < std::rvalue_ref < FormalRTESymbol > > elements;

	std::rvalue_ref < FormalRTESymbolAlphabet > ret ( new FormalRTESymbolAlphabet ( symbol ) );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		elements.push_back ( parseFormalRTESymbol ( input ) );

	for ( std::rvalue_ref < FormalRTESymbol > & element : elements )
		ret->appendElement ( element );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "symbol" );

	return ret;
}

std::rvalue_ref < FormalRTESymbolSubst > RTEFromXMLParser::parseFormalRTESymbolSubst ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbol" );
	alphabet::RankedSymbol symbol = alib::xmlApi < alphabet::RankedSymbol >::parse ( input );

	std::rvalue_ref < FormalRTESymbolSubst > ret ( new FormalRTESymbolSubst ( symbol ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substSymbol" );

	return ret;
}

} /* namespace rte */
