#ifndef FORMAL_RTE_ELEMENT_H_
#define FORMAL_RTE_ELEMENT_H_

#include "../../alphabet/Symbol.h"
#include "../common/RTEAlphabet.h"
#include <XmlApi.hpp>
#include <set>
#include <tree>

namespace rte {

class FormalRTEAlternation;
class FormalRTEIteration;
class FormalRTESubstitution;
class FormalRTESymbol;
class FormalRTESymbolAlphabet;
class FormalRTESymbolSubst;
class FormalRTEEmpty;

class FormalRTEElement;

class FormalRTEElementVisitor {
public:
	virtual void Visit ( void *, const FormalRTEAlternation & alternation ) const = 0;
	virtual void Visit ( void *, const FormalRTEIteration & iteration ) const = 0;
	virtual void Visit ( void *, const FormalRTESubstitution & substitution ) const = 0;
	virtual void Visit ( void *, const FormalRTESymbolAlphabet & symbol ) const = 0;
	virtual void Visit ( void *, const FormalRTESymbolSubst & symbol ) const = 0;
	virtual void Visit ( void *, const FormalRTEEmpty & empty ) const = 0;
};

class FormalRTEElement : public alib::CommonBase < FormalRTEElement >, public std::BaseNode < FormalRTEElement > {
public:
	virtual void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const = 0;

	/**
	 * Traverses the regexp tree looking if particular Symbol is used in the regexp.
	 *
	 * @param symbol to test if used in regexp element
	 * @return true if symbol is used by the element and its successor
	 */
	virtual bool testSymbol ( const alphabet::RankedSymbol & symbol ) const = 0;

	/**
	 * Traverses the regexp tree computing minimal alphabet needed by regexp
	 *
	 * @param alphabet All alphabet symbols encountered are added into this set
	 * @return true if symbol is used by the element and its successor
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const = 0;

	/**
	 * @copydoc RankedNode::computeMinimalAlphabet()
	 */
	std::pair < std::set < alphabet::RankedSymbol >, std::set < alphabet::RankedSymbol > > computeMinimalAlphabets ( ) const;

	/**
	 * Traverses the regexp tree and checks whether all symbols in the regexp tree are in the alphabet
	 *
	 * @param alphabet
	 * @return true if symbols in the regexp are in the alphabet, false otherwise
	 */
	virtual bool checkAlphabet ( const std::set < alphabet::RankedSymbol > & alphabetF, const std::set < alphabet::RankedSymbol > & alphabetK ) const = 0;
};

} /* namespace rte */

namespace alib {

template < >
struct xmlApi < rte::FormalRTEElement > {
	static std::rvalue_ref < rte::FormalRTEElement > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, const rte::FormalRTEElement & data );
};

} /* namespace alib */

#endif /* FORMAL_RTE_ELEMENT_H_ */
