#include "FormalRTESubstitution.h"
#include <exception/CommonException.h>
#include <sstream>

namespace rte {

FormalRTESubstitution::FormalRTESubstitution ( FormalRTEElement && left, FormalRTEElement && right, FormalRTESymbolSubst substitutionSymbol ) : BinaryNode < std::smart_ptr < FormalRTEElement >, std::smart_ptr < FormalRTEElement >, FormalRTESubstitution > ( std::smart_ptr < FormalRTEElement > ( std::move ( left ).plunder ( ) ), std::smart_ptr < FormalRTEElement > ( std::move ( right ).plunder ( ) ) ), substitutionSymbol ( std::smart_ptr < FormalRTESymbolSubst > ( std::move ( substitutionSymbol ).plunder ( ) ) ) {
}

FormalRTESubstitution::FormalRTESubstitution ( const FormalRTEElement & left, const FormalRTEElement & right, FormalRTESymbolSubst substitutionSymbol ) : FormalRTESubstitution ( std::move_copy ( left ), std::move_copy ( right ), std::move ( substitutionSymbol ) ) {
}

const FormalRTEElement & FormalRTESubstitution::getLeftElement ( ) const {
	return * getLeft ( );
}

const FormalRTEElement & FormalRTESubstitution::getRightElement ( ) const {
	return * getRight ( );
}

const FormalRTESymbolSubst & FormalRTESubstitution::getSubstitutionSymbol ( ) const {
	return * substitutionSymbol;
}

FormalRTEElement & FormalRTESubstitution::getLeftElement ( ) {
	return * getLeft ( );
}

FormalRTEElement & FormalRTESubstitution::getRightElement ( ) {
	return * getRight ( );
}

FormalRTESymbolSubst & FormalRTESubstitution::getSubstitutionSymbol ( ) {
	return * substitutionSymbol;
}

void FormalRTESubstitution::setLeftElement ( const FormalRTEElement & element ) {
	setLeftElement ( std::move_copy ( element ) );
}

void FormalRTESubstitution::setLeftElement ( FormalRTEElement && element ) {
	setLeft ( std::smart_ptr < FormalRTEElement > ( std::move ( element ).plunder ( ) ) );
}

void FormalRTESubstitution::setRightElement ( const FormalRTEElement & element ) {
	setRightElement ( std::move_copy ( element ) );
}

void FormalRTESubstitution::setRightElement ( FormalRTEElement && element ) {
	setRight ( std::smart_ptr < FormalRTEElement > ( std::move ( element ).plunder ( ) ) );
}

void FormalRTESubstitution::setSubstitutionSymbol ( FormalRTESymbolSubst symbol ) {
	substitutionSymbol = std::smart_ptr < FormalRTESymbolSubst > ( std::move ( symbol ).plunder ( ) );
}

FormalRTESubstitution * FormalRTESubstitution::clone ( ) const {
	return new FormalRTESubstitution ( * this );
}

FormalRTESubstitution * FormalRTESubstitution::plunder ( ) && {
	return new FormalRTESubstitution ( std::move ( * this ) );
}

int FormalRTESubstitution::compare ( const FormalRTESubstitution & other ) const {
	auto first = std::tie ( getLeftElement ( ), getRightElement ( ), getSubstitutionSymbol ( ) );
	auto second = std::tie ( other.getLeftElement ( ), other.getRightElement ( ), other.getSubstitutionSymbol ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

void FormalRTESubstitution::operator >>( std::ostream & out ) const {
	out << "(FormalRTESubstitution";
	out << " " << getLeftElement ( );
	out << " " << * substitutionSymbol;
	out << " " << getRightElement ( );
	out << ")";
}

bool FormalRTESubstitution::testSymbol ( const alphabet::RankedSymbol & symbol ) const {
	if ( getLeftElement ( ).testSymbol ( symbol ) ) return true;

	if ( getRightElement ( ).testSymbol ( symbol ) ) return true;

	if ( substitutionSymbol->getSymbol ( ) == symbol ) return true;

	return false;
}

void FormalRTESubstitution::computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const {
	alphabetK.insert ( substitutionSymbol->getSymbol ( ) );
	getLeftElement ( ).computeMinimalAlphabet ( alphabetF, alphabetK );
	getRightElement ( ).computeMinimalAlphabet ( alphabetF, alphabetK );
}

bool FormalRTESubstitution::checkAlphabet ( const std::set < alphabet::RankedSymbol > & alphabetF, const std::set < alphabet::RankedSymbol > & alphabetK ) const {
	return alphabetK.count ( getSubstitutionSymbol ( ).getSymbol ( ) ) > 0 && getLeftElement ( ).checkAlphabet ( alphabetF, alphabetK ) && getRightElement ( ).checkAlphabet ( alphabetF, alphabetK );
}

FormalRTESubstitution::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
