#ifndef FORMAL_RTE_SUBST_SYMBOL_H_
#define FORMAL_RTE_SUBST_SYMBOL_H_

#include "FormalRTE.h"
#include "FormalRTESymbol.h"

namespace rte {

/**
 * Represents constant symbol (element of alphabet K) in the regular expression. Contains name of the symbol.
 */
class FormalRTESymbolSubst : public FormalRTESymbol, public std::NullaryNode < FormalRTESymbolSubst > {
public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTESymbolSubst ( alphabet::RankedSymbol symbol );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTESymbolSubst * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTESymbolSubst * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::RankedSymbol & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < alphabet::RankedSymbol > & alphabetF, const std::set < alphabet::RankedSymbol > & alphabetK ) const;

	virtual int compare ( const FormalRTESymbolSubst & ) const;

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	/**
	 * Returns the string representation of RTE Substitution Symbol.
	 */
	virtual explicit operator std::string ( ) const;

	friend class FormalRTEIteration;
	friend class FormalRTESubstitution;
};

} /* namespace rte */

#endif /* FORMAL_RTE_SUBST_SYMBOL_H_ */
