/*
 * StringFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_FROM_XML_PARSER_H_
#define STRING_FROM_XML_PARSER_H_

#include <vector>
#include <set>
#include <deque>
#include <sax/Token.h>
#include <sax/FromXMLParserHelper.h>
#include <XmlApi.hpp>

namespace string {

/**
 * Parser used to get String from XML parsed into list of tokens.
 */
class StringFromXMLParser {
public:
	template < class SymbolType >
	static std::vector<SymbolType> parseContent(std::deque<sax::Token>::iterator& input);
	template < class SymbolType >
	static std::set<SymbolType> parseAlphabet(std::deque<sax::Token>::iterator& input);
};

template < class SymbolType >
std::set<SymbolType> StringFromXMLParser::parseAlphabet(std::deque<sax::Token>::iterator& input) {
	std::set<SymbolType> alphabet;
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "alphabet");
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		alphabet.insert(alib::xmlApi<SymbolType>::parse(input));
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "alphabet");
	return alphabet;
}

template < class SymbolType >
std::vector<SymbolType> StringFromXMLParser::parseContent(std::deque<sax::Token>::iterator& input) {
	std::vector<SymbolType> data;
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "content");
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		data.push_back(alib::xmlApi<SymbolType>::parse(input));
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "content");
	return data;
}

} /* namespace string */

#endif /* STRING_FROM_XML_PARSER_H_ */
