#include <list>
#include "ContainerTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "object/Object.h"
#include "container/ObjectsSet.h"
#include "container/ObjectsVariant.h"
#include "container/ObjectsTree.h"
#include "alphabet/LabeledSymbol.h"
#include "primitive/Integer.h"
#include "primitive/String.h"

#include "factory/XmlDataFactory.hpp"

#define CPPUNIT_IMPLY( x, y )			CPPUNIT_ASSERT ( !( x ) || ( y ) )
#define CPPUNIT_EXCLUSIVE_OR( x, y )	CPPUNIT_ASSERT ( ( !( x ) && ( y ) ) || ( ( x ) && !( y ) ) )

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( ContainerTest, "container" );
CPPUNIT_TEST_SUITE_REGISTRATION ( ContainerTest );

void ContainerTest::setUp ( ) {
}

void ContainerTest::tearDown ( ) {
}

void ContainerTest::testXMLParser ( ) {

	alib::Object tmp ( alphabet::LabeledSymbol ( "1" ) );

	container::ObjectsSet set;

	set.insert ( tmp );

	alib::Object object ( set );

	{
		std::string tmp = alib::XmlDataFactory::toString ( object );
		alib::Object object2 = alib::XmlDataFactory::fromString < alib::Object > ( tmp );

		CPPUNIT_ASSERT ( object == object2 );

		std::set < alphabet::LabeledSymbol > concrete = alib::XmlDataFactory::fromString < std::set < alphabet::LabeledSymbol > > ( tmp );
		std::string tmp2 = alib::XmlDataFactory::toString ( concrete );

		CPPUNIT_ASSERT ( tmp == tmp2 );
	}
}

void ContainerTest::testVariantParsing ( ) {
	{
		std::string string = "<String>aaa</String>";
		std::variant < int, std::string > object = alib::XmlDataFactory::fromString < std::variant < int, std::string > > ( string );

		CPPUNIT_ASSERT ( "aaa" == object.get < std::string > ( ) );

		std::string string2 = alib::XmlDataFactory::toString ( object );

		std::variant < int, std::string > object2 = alib::XmlDataFactory::fromString < std::variant < int, std::string > > ( string2 );

		CPPUNIT_ASSERT ( object == object2 );
	}
}

void ContainerTest::testTreeParsing ( ) {
	{
		std::tree < int > t ( 1 );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::tree < int > t2 = alib::XmlDataFactory::fromString < std::tree < int > > ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		std::tree < int > t ( 1, std::tree < int > ( 2 ), std::tree < int > ( 4 ) );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::tree < int > t2 = alib::XmlDataFactory::fromString < std::tree < int > > ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		std::tree < int > t ( 1, std::tree < int > ( 2, std::tree < int > ( 3 ) ), std::tree < int > ( 4 ) );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::tree < int > t2 = alib::XmlDataFactory::fromString < std::tree < int > > ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
}
