/*
 * AutomatonFromStringParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_FROM_STRING_PARSER_H_
#define AUTOMATON_FROM_STRING_PARSER_H_

#include "AutomatonFromStringLexer.h"

#include "automaton/AutomatonFeatures.h"
#include "string/StringFeatures.h"
#include "alphabet/SymbolFeatures.h"
#include "label/LabelFeatures.h"

#include <set>
#include <vector>
#include <tuple>
#include <variant>

namespace alib {

template<typename T>
struct stringApi;

} /* namespace alib */

namespace automaton {

class AutomatonFromStringParser {
public:
	AutomatonFromStringParser() {}

private:
	AutomatonFromStringLexer m_AutomatonLexer;

	void initialFinalState(std::istream& input, bool& initial, bool& final) const;

	void parseEpsilonNFATransition(std::istream& input, std::set<label::Label>& states, const std::vector<std::variant<string::Epsilon < >, alphabet::Symbol>>& symbols, label::Label*& initialState, std::set<label::Label>& finalStates, std::set<std::tuple<label::Label, std::variant<string::Epsilon < >, alphabet::Symbol>, label::Label>>& transitionFunction) const;
	void parseMultiInitialStateNFATransition(std::istream& input, MultiInitialStateNFA < > & res, const std::vector<alphabet::Symbol>& symbols) const;
	void parseNFATransition(std::istream& input, std::set<label::Label>& states, const std::vector<alphabet::Symbol>& symbols, label::Label*& initialState, std::set<label::Label>& finalStates, std::set<std::tuple<label::Label, alphabet::Symbol, label::Label>>& transitionFunction) const;
	void parseDFATransition(std::istream& input, std::set<label::Label>& states, const std::vector<alphabet::Symbol>& symbols, label::Label*& initialState, std::set<label::Label>& finalStates, std::set<std::tuple<label::Label, alphabet::Symbol, label::Label>>& transitionFunction) const;

	Automaton parseAutomaton(std::istream& input) const;
	Automaton parseAutomaton(std::istream& input, const std::set<FEATURES>& features) const;

	EpsilonNFA<> parseEpsilonNFA(std::istream& input) const;
	NFA<> parseNFA(std::istream& input) const;
	MultiInitialStateNFA<> parseMultiInitialStateNFA(std::istream& input) const;
	DFA<> parseDFA(std::istream& input) const;

	template<typename T> friend struct alib::stringApi;
};

} /* namespace automaton */

#endif /* AUTOMATON_FROM_STRING_PARSER_H_ */
