/*
 * GrammarFromStringParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef GRAMMAR_FROM_STRING_PARSER_H_
#define GRAMMAR_FROM_STRING_PARSER_H_

#include "GrammarFromStringLexer.h"
#include "grammar/GrammarFeatures.h"
#include "alphabet/Symbol.h"
#include <set>
#include <vector>
#include <map>

namespace alib {

template<typename T>
struct stringApi;

} /* namespace alib */

namespace grammar {

class GrammarFromStringParser {
public:
	GrammarFromStringParser() {}

private:
	GrammarFromStringLexer m_GrammarLexer;

	std::set<alphabet::Symbol> parseSet(std::istream& input) const;

	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> parseCFLikeRules(std::istream& input) const;
	template<class T> T parseCFLikeGrammar(std::istream& input) const;

	std::map<std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol>>> parseCSLikeRules(std::istream& input) const;
	template<class T> T parseCSLikeGrammar(std::istream& input) const;

	std::map<std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>, std::set<std::vector<alphabet::Symbol>>> parsePreservingCSLikeRules(std::istream& input) const;
	template<class T> T parsePreservingCSLikeGrammar(std::istream& input) const;

	Grammar parseGrammar(std::istream& input) const;
	Grammar parseGrammar(std::istream& input, const std::set<FEATURES>& features) const;

	RightRG < > parseRightRG(std::istream& input) const;
	LeftRG < > parseLeftRG(std::istream& input) const;
	RightLG < > parseRightLG(std::istream& input) const;
	LeftLG < > parseLeftLG(std::istream& input) const;

	LG < > parseLG(std::istream& input) const;
	CFG < > parseCFG(std::istream& input) const;
	EpsilonFreeCFG < > parseEpsilonFreeCFG(std::istream& input) const;
	GNF < > parseGNF(std::istream& input) const;
	CNF < > parseCNF(std::istream& input) const;

	NonContractingGrammar < > parseNonContractingGrammar(std::istream& input) const;
	CSG < > parseCSG(std::istream& input) const;
	ContextPreservingUnrestrictedGrammar < > parseContextPreservingUnrestrictedGrammar(std::istream& input) const;
	UnrestrictedGrammar < > parseUnrestrictedGrammar(std::istream& input) const;

	template<typename T> friend struct alib::stringApi;
};

} /* namespace grammar */

#endif /* GRAMMAR_FROM_STRING_PARSER_H_ */
