/*
 * TreeToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "TreeToStringComposer.h"
#include <tree/ranked/RankedTree.h>
#include <tree/ranked/RankedPattern.h>
#include <tree/unranked/UnrankedTree.h>
#include <tree/unranked/UnrankedPattern.h>

#include "../StringApi.hpp"

namespace tree {

void TreeToStringComposer::compose ( std::ostream & out, const RankedTree & tree ) {
	compose ( out, tree.getContent ( ) );
}

TreeToStringComposer::RegistratorWrapper < void, RankedTree > StringToStringComposerRankedTree = TreeToStringComposer::RegistratorWrapper < void, RankedTree > ( TreeToStringComposer::compose );

void TreeToStringComposer::compose ( std::ostream & out, const std::tree < alphabet::RankedSymbol > & node ) {
	alib::stringApi < alphabet::Symbol >::compose ( out, node.getData ( ).getSymbol ( ) );

	out << std::to_string ( node.getData ( ).getRank ( ).getData ( ) );

	for ( const std::tree < alphabet::RankedSymbol > & child : node ) {
		out << " ";
		compose ( out, child );
	}
}

void TreeToStringComposer::compose ( std::ostream & out, const RankedPattern & tree ) {
	compose ( out, tree.getSubtreeWildcard ( ), tree.getContent ( ) );
}

TreeToStringComposer::RegistratorWrapper < void, RankedPattern > StringToStringComposerRankedPattern = TreeToStringComposer::RegistratorWrapper < void, RankedPattern > ( TreeToStringComposer::compose );

void TreeToStringComposer::compose ( std::ostream & out, const alphabet::RankedSymbol & subtreeWildcard, const std::tree < alphabet::RankedSymbol > & node ) {
	if ( node.getData ( ) == subtreeWildcard ) {
		out << "#S";
	} else {
		alib::stringApi < alphabet::Symbol >::compose ( out, node.getData ( ).getSymbol ( ) );

		out << std::to_string ( node.getData ( ).getRank ( ).getData ( ) );

		for ( const std::tree < alphabet::RankedSymbol > & child : node ) {
			out << " ";
			compose ( out, subtreeWildcard, child );
		}
	}
}

void TreeToStringComposer::compose ( std::ostream & out, const UnrankedTree & tree ) {
	compose ( out, tree.getContent ( ) );
}

TreeToStringComposer::RegistratorWrapper < void, UnrankedTree > StringToStringComposerUnrankedTree = TreeToStringComposer::RegistratorWrapper < void, UnrankedTree > ( TreeToStringComposer::compose );

void TreeToStringComposer::compose ( std::ostream & out, const std::tree < alphabet::Symbol > & node ) {
	alib::stringApi < alphabet::Symbol >::compose ( out, node.getData ( ) );

	for ( const std::tree < alphabet::Symbol > & child : node ) {
		out << " ";
		compose ( out, child );
	}

	out << " |";
}

void TreeToStringComposer::compose ( std::ostream & out, const UnrankedPattern & tree ) {
	compose ( out, tree.getSubtreeWildcard ( ), tree.getContent ( ) );
}

TreeToStringComposer::RegistratorWrapper < void, UnrankedPattern > StringToStringComposerUnrankedPattern = TreeToStringComposer::RegistratorWrapper < void, UnrankedPattern > ( TreeToStringComposer::compose );

void TreeToStringComposer::compose ( std::ostream & out, const alphabet::Symbol & subtreeWildcard, const std::tree < alphabet::Symbol > & node ) {
	if ( node.getData ( ) == subtreeWildcard ) {
		out << "#S |";
	} else {
		alib::stringApi < alphabet::Symbol >::compose ( out, node.getData ( ) );

		for ( const std::tree < alphabet::Symbol > & child : node ) {
			out << " ";
			compose ( out, subtreeWildcard, child );
		}

		out << " |";
	}
}

void TreeToStringComposer::compose ( std::ostream & out, const Tree & tree ) {
	dispatch ( out, tree.getData ( ) );
}

} /* namespace tree */
