/*
 * TreeToStringComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnciek
 */

#ifndef TREE_TO_STRING_COMPOSER_H_
#define TREE_TO_STRING_COMPOSER_H_

#include <ostream>
#include <tree>
#include <core/multipleDispatch.hpp>
#include "tree/Tree.h"
#include "tree/TreeFeatures.h"

#include "alphabet/RankedSymbol.h"

namespace tree {

/**
 * This class contains methods to print XML representation of tree to the output stream.
 */
class TreeToStringComposer : public std::SingleDispatchFirstStaticParam < TreeToStringComposer, void, std::ostream &, const TreeBase &> {
	static void compose ( std::ostream &, const std::tree < alphabet::RankedSymbol > & node );
	static void compose ( std::ostream &, const alphabet::RankedSymbol & subtreeWildcard, const std::tree < alphabet::RankedSymbol > & node );
	static void compose ( std::ostream &, const std::tree < alphabet::Symbol > & node );
	static void compose ( std::ostream &, const alphabet::Symbol & subtreeWildcard, const std::tree < alphabet::Symbol > & node );

public:
	static void compose ( std::ostream &, const RankedTree & tree );
	static void compose ( std::ostream &, const RankedPattern & tree );
	static void compose ( std::ostream &, const UnrankedTree & tree );
	static void compose ( std::ostream &, const UnrankedPattern & tree );

	/**
	 * Prints XML representation of String to the output stream.
	 * @param tree String to print
	 * @param out output stream to which print the String
	 */
	static void compose ( std::ostream & output, const Tree & tree );

};

} /* namespace tree */

#endif /* TREE_TO_STRING_COMPOSER_H_ */
